'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (_ref) {
    var types = _ref.types;

    var autoRequire = undefined,
        requireNode = undefined,
        requireNormalizer = undefined;

    function buildNodeExpr(tagExpr) {
        return types.callExpression(autoRequire ? types.identifier(NODE_BUILDER) : types.memberExpression(types.identifier(VIDOM), types.identifier('node')), [tagExpr]);
    }

    function buildChildrenExpr(children, prevExpr) {
        var normalizedChildren = normalizeChildren(children);

        return normalizedChildren ? types.callExpression(types.memberExpression(prevExpr, types.identifier('children')), [normalizedChildren]) : prevExpr;
    }

    function buildAttrsExpr(attrs, prevExpr, file) {
        var res = prevExpr,
            attrList = [],
            objList = [],
            attrsExpr = undefined,
            nsExpr = undefined,
            keyExpr = undefined,
            htmlExpr = undefined,
            domRefExpr = undefined,
            pushAttrs = function pushAttrs() {
            if (!attrList.length) {
                return;
            }

            objList.push(types.objectExpression(attrList));
            attrList = [];
        };

        attrs.forEach(function (attr) {
            if (types.isJSXSpreadAttribute(attr)) {
                pushAttrs();
                objList.push(attr.argument);
            } else {
                switch (attr.name.name) {
                    case 'xmlns':
                        nsExpr = getValueExpr(attr.value);
                        break;

                    case 'key':
                        keyExpr = getValueExpr(attr.value);
                        break;

                    case 'html':
                        htmlExpr = getValueExpr(attr.value);
                        break;

                    case 'dom-ref':
                        domRefExpr = getValueExpr(attr.value);
                        break;

                    default:
                        attrList.push(types.objectProperty(types.stringLiteral(attr.name.name), types.isJSXExpressionContainer(attr.value) ? attr.value.expression : attr.value || types.booleanLiteral(true)));
                }
            }
        });

        pushAttrs();

        if (objList.length) {
            if (objList.length === 1) {
                attrsExpr = objList[0];
            } else {
                if (!types.isObjectExpression(objList[0])) {
                    objList.unshift(types.objectExpression([]));
                }

                attrsExpr = types.callExpression(file.addHelper('extends'), objList);
            }
        }

        if (domRefExpr) {
            res = types.callExpression(types.memberExpression(types.identifier('this'), types.identifier('setDomRef')), [domRefExpr, res]);
        }

        if (nsExpr) {
            res = types.callExpression(types.memberExpression(res, types.identifier('ns')), [nsExpr]);
        }

        if (keyExpr) {
            res = types.callExpression(types.memberExpression(res, types.identifier('key')), [keyExpr]);
        }

        if (attrsExpr) {
            res = types.callExpression(types.memberExpression(res, types.identifier('attrs')), [attrsExpr]);
        }

        if (htmlExpr) {
            res = types.callExpression(types.memberExpression(res, types.identifier('html')), [htmlExpr]);
        }

        return res;
    }

    function getValueExpr(value) {
        return types.isJSXExpressionContainer(value) ? value.expression : value;
    }

    function normalizeChildren(children) {
        var normalizeInRuntime = false,
            res = children.reduce(function (acc, child) {
            if (types.isJSXText(child)) {
                child = cleanJSXText(child);

                if (child) {
                    if (children.length > 1) {
                        acc.push(types.callExpression(types.memberExpression(buildNodeExpr(types.stringLiteral('span')), types.identifier('children')), [child]));
                    } else {
                        acc.push(child);
                    }
                }
            } else if (types.isJSXExpressionContainer(child)) {
                if (!types.isJSXEmptyExpression(child.expression)) {
                    normalizeInRuntime = true;
                    requireNormalizer = true;
                    acc.push(child.expression);
                }
            } else {
                acc.push(child);
            }

            return acc;
        }, []);

        return normalizeInRuntime ? types.callExpression(autoRequire ? types.identifier(CHILDREN_NORMALIZER) : types.memberExpression(types.identifier(VIDOM), types.identifier('normalizeChildren')), [res.length > 1 ? types.arrayExpression(res) : res[0]]) : res.length > 1 ? types.arrayExpression(res) : res[0];
    }

    function cleanJSXText(node) {
        var lines = node.value.split(/\r\n|\n|\r/);
        var lastNonEmptyLine = 0;

        lines.forEach(function (line, i) {
            if (line.match(/[^ \t]/)) {
                lastNonEmptyLine = i;
            }
        });

        var str = '';

        lines.forEach(function (line, i) {
            var isFirstLine = i === 0,
                isLastLine = i === lines.length - 1,
                isLastNonEmptyLine = i === lastNonEmptyLine;

            var trimmedLine = line.replace(/\t/g, ' ');

            if (!isFirstLine) {
                trimmedLine = trimmedLine.replace(/^[ ]+/, '');
            }

            if (!isLastLine) {
                trimmedLine = trimmedLine.replace(/[ ]+$/, '');
            }

            if (trimmedLine) {
                if (!isLastNonEmptyLine) {
                    trimmedLine += ' ';
                }

                str += trimmedLine;
            }
        });

        if (str) {
            return types.stringLiteral(str);
        }
    }

    return {
        inherits: _babelPluginSyntaxJsx2.default,
        visitor: {
            JSXElement: function JSXElement(path, file) {
                requireNode = true;

                var node = path.node,
                    name = node.openingElement.name.name,
                    attrs = node.openingElement.attributes,
                    children = node.children;

                var res = buildNodeExpr(name === name.toLowerCase() ? types.stringLiteral(name) : node.openingElement.name);

                if (attrs.length) {
                    res = buildAttrsExpr(attrs, res, file);
                }

                if (children.length) {
                    res = buildChildrenExpr(children, res);
                }

                path.replaceWith(res);
            },

            Program: {
                enter: function enter(_, _ref2) {
                    var opts = _ref2.opts;

                    autoRequire = opts.autoRequire !== false;
                    requireNode = false;
                    requireNormalizer = false;
                },
                exit: function exit(path) {
                    if (!requireNode) {
                        return;
                    }

                    if (autoRequire) {
                        var requireExpr = types.callExpression(types.identifier('require'), [types.stringLiteral(VIDOM)]);

                        path.node.body.unshift(types.variableDeclaration('var', [types.variableDeclarator(types.identifier(NODE_BUILDER), types.memberExpression(requireExpr, types.identifier('node')))].concat(requireNormalizer ? types.variableDeclarator(types.identifier(CHILDREN_NORMALIZER), types.memberExpression(requireExpr, types.identifier('normalizeChildren'))) : [])));
                    }
                }
            }
        }
    };
};

var _babelPluginSyntaxJsx = require('babel-plugin-syntax-jsx');

var _babelPluginSyntaxJsx2 = _interopRequireDefault(_babelPluginSyntaxJsx);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var VIDOM = 'vidom',
    NODE_BUILDER = '__vnode__',
    CHILDREN_NORMALIZER = '__vnormalizer__';